from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor
RESOURCE_TYPE_ACCESS: ResourceType
RESOURCE_TYPE_CLIENT_SECRET: ResourceType
RESOURCE_TYPE_CONFIG: ResourceType
RESOURCE_TYPE_DATA: ResourceType
RESOURCE_TYPE_DATASET: ResourceType
RESOURCE_TYPE_DEPLOYMENT_SETTINGS: ResourceType
RESOURCE_TYPE_FEATURES: ResourceType
RESOURCE_TYPE_MATERIALIZATION_STATE: ResourceType
RESOURCE_TYPE_MEMBERS: ResourceType
RESOURCE_TYPE_METRIC: ResourceType
RESOURCE_TYPE_ORGANIZATION: ResourceType
RESOURCE_TYPE_PLATFORM: ResourceType
RESOURCE_TYPE_PRINCIPAL_GROUP: ResourceType
RESOURCE_TYPE_PRINCIPAL_GROUPS_MEMBERSHIPS: ResourceType
RESOURCE_TYPE_SECRET: ResourceType
RESOURCE_TYPE_SECRET_SCOPE: ResourceType
RESOURCE_TYPE_SERVICE_ACCOUNT: ResourceType
RESOURCE_TYPE_UNSPECIFIED: ResourceType
RESOURCE_TYPE_USER: ResourceType
RESOURCE_TYPE_WORKSPACE: ResourceType

class ResourceType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
