# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/service_account__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.auditlog import metadata__client_pb2 as tecton__proto_dot_auditlog_dot_metadata____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n/tecton_proto/data/service_account__client.proto\x12\x11tecton_proto.data\x1a,tecton_proto/auditlog/metadata__client.proto\"\x98\x01\n\x0fNewClientSecret\x12$\n\tsecret_id\x18\x01 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x08secretId\x12&\n\ncreated_at\x18\x02 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\tcreatedAt\x12\x1f\n\x06status\x18\x03 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x06status\x12\x16\n\x06secret\x18\x04 \x01(\tR\x06secret\"\xd0\x01\n\x12MaskedClientSecret\x12$\n\tsecret_id\x18\x01 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x08secretId\x12&\n\ncreated_at\x18\x02 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\tcreatedAt\x12&\n\nupdated_at\x18\x03 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\tupdatedAt\x12\x1f\n\x06status\x18\x04 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x06status\x12#\n\rmasked_secret\x18\x05 \x01(\tR\x0cmaskedSecret*\xbe\x01\n\x1dServiceAccountCredentialsType\x12\x30\n,SERVICE_ACCOUNT_CREDENTIALS_TYPE_UNSPECIFIED\x10\x00\x12,\n(SERVICE_ACCOUNT_CREDENTIALS_TYPE_API_KEY\x10\x01\x12=\n9SERVICE_ACCOUNT_CREDENTIALS_TYPE_OAUTH_CLIENT_CREDENTIALS\x10\x02\x42;\n\x0f\x63om.tecton.dataP\x01Z&github.com/tecton-ai/tecton_proto/data')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.data.service_account__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.dataP\001Z&github.com/tecton-ai/tecton_proto/data'
  _NEWCLIENTSECRET.fields_by_name['secret_id']._options = None
  _NEWCLIENTSECRET.fields_by_name['secret_id']._serialized_options = b'\352\231\275F\002\010\002'
  _NEWCLIENTSECRET.fields_by_name['created_at']._options = None
  _NEWCLIENTSECRET.fields_by_name['created_at']._serialized_options = b'\352\231\275F\002\010\002'
  _NEWCLIENTSECRET.fields_by_name['status']._options = None
  _NEWCLIENTSECRET.fields_by_name['status']._serialized_options = b'\352\231\275F\002\010\002'
  _MASKEDCLIENTSECRET.fields_by_name['secret_id']._options = None
  _MASKEDCLIENTSECRET.fields_by_name['secret_id']._serialized_options = b'\352\231\275F\002\010\002'
  _MASKEDCLIENTSECRET.fields_by_name['created_at']._options = None
  _MASKEDCLIENTSECRET.fields_by_name['created_at']._serialized_options = b'\352\231\275F\002\010\002'
  _MASKEDCLIENTSECRET.fields_by_name['updated_at']._options = None
  _MASKEDCLIENTSECRET.fields_by_name['updated_at']._serialized_options = b'\352\231\275F\002\010\002'
  _MASKEDCLIENTSECRET.fields_by_name['status']._options = None
  _MASKEDCLIENTSECRET.fields_by_name['status']._serialized_options = b'\352\231\275F\002\010\002'
  _SERVICEACCOUNTCREDENTIALSTYPE._serialized_start=483
  _SERVICEACCOUNTCREDENTIALSTYPE._serialized_end=673
  _NEWCLIENTSECRET._serialized_start=117
  _NEWCLIENTSECRET._serialized_end=269
  _MASKEDCLIENTSECRET._serialized_start=272
  _MASKEDCLIENTSECRET._serialized_end=480
# @@protoc_insertion_point(module_scope)
