# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/args/server_group__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.args import basic_info__client_pb2 as tecton__proto_dot_args_dot_basic__info____client__pb2
from tecton_proto.args import diff_options__client_pb2 as tecton__proto_dot_args_dot_diff__options____client__pb2
from tecton_proto.common import id__client_pb2 as tecton__proto_dot_common_dot_id____client__pb2
from tecton_proto.common import scaling_config__client_pb2 as tecton__proto_dot_common_dot_scaling__config____client__pb2
from tecton_proto.common import server_group_type__client_pb2 as tecton__proto_dot_common_dot_server__group__type____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n,tecton_proto/args/server_group__client.proto\x12\x11tecton_proto.args\x1a*tecton_proto/args/basic_info__client.proto\x1a,tecton_proto/args/diff_options__client.proto\x1a$tecton_proto/common/id__client.proto\x1a\x30tecton_proto/common/scaling_config__client.proto\x1a\x33tecton_proto/common/server_group_type__client.proto\"\x8c\x07\n\x0fServerGroupArgs\x12?\n\x0fserver_group_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\rserverGroupId\x12\x37\n\x04info\x18\x02 \x01(\x0b\x32\x1c.tecton_proto.args.BasicInfoB\x05\x92M\x02\x10\x01R\x04info\x12P\n\x11server_group_type\x18\x03 \x01(\x0e\x32$.tecton_proto.common.ServerGroupTypeR\x0fserverGroupType\x12s\n\x1btransform_server_group_args\x18\x08 \x01(\x0b\x32+.tecton_proto.args.TransformServerGroupArgsB\x05\x92M\x02\x10\x01H\x00R\x18transformServerGroupArgs\x12m\n\x19\x66\x65\x61ture_server_group_args\x18\x0b \x01(\x0b\x32).tecton_proto.args.FeatureServerGroupArgsB\x05\x92M\x02\x10\x01H\x00R\x16\x66\x65\x61tureServerGroupArgs\x12.\n\x0fprevent_destroy\x18\t \x01(\x08\x42\x05\x92M\x02\x08\x01R\x0epreventDestroy\x12P\n\x07options\x18\n \x03(\x0b\x32/.tecton_proto.args.ServerGroupArgs.OptionsEntryB\x05\x92M\x02\x08\x01R\x07options\x12^\n\x12\x61utoscaling_config\x18\x0c \x01(\x0b\x32&.tecton_proto.common.AutoscalingConfigB\x05\x92M\x02\x08\x01H\x01R\x11\x61utoscalingConfig\x12t\n\x1aprovisioned_scaling_config\x18\r \x01(\x0b\x32-.tecton_proto.common.ProvisionedScalingConfigB\x05\x92M\x02\x08\x01H\x01R\x18provisionedScalingConfig\x1a:\n\x0cOptionsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x42\x0b\n\ttype_argsB\x10\n\x0escaling_configJ\x04\x08\x04\x10\x05J\x04\x08\x05\x10\x06J\x04\x08\x06\x10\x07J\x04\x08\x07\x10\x08\"<\n\x18TransformServerGroupArgs\x12 \n\x0b\x65nvironment\x18\x01 \x01(\tR\x0b\x65nvironment\"\x18\n\x16\x46\x65\x61tureServerGroupArgs\"k\n\x14ServerGroupReference\x12?\n\x0fserver_group_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\rserverGroupId\x12\x12\n\x04name\x18\x02 \x01(\tR\x04nameB;\n\x0f\x63om.tecton.argsP\x01Z&github.com/tecton-ai/tecton_proto/args')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.args.server_group__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.argsP\001Z&github.com/tecton-ai/tecton_proto/args'
  _SERVERGROUPARGS_OPTIONSENTRY._options = None
  _SERVERGROUPARGS_OPTIONSENTRY._serialized_options = b'8\001'
  _SERVERGROUPARGS.fields_by_name['info']._options = None
  _SERVERGROUPARGS.fields_by_name['info']._serialized_options = b'\222M\002\020\001'
  _SERVERGROUPARGS.fields_by_name['transform_server_group_args']._options = None
  _SERVERGROUPARGS.fields_by_name['transform_server_group_args']._serialized_options = b'\222M\002\020\001'
  _SERVERGROUPARGS.fields_by_name['feature_server_group_args']._options = None
  _SERVERGROUPARGS.fields_by_name['feature_server_group_args']._serialized_options = b'\222M\002\020\001'
  _SERVERGROUPARGS.fields_by_name['prevent_destroy']._options = None
  _SERVERGROUPARGS.fields_by_name['prevent_destroy']._serialized_options = b'\222M\002\010\001'
  _SERVERGROUPARGS.fields_by_name['options']._options = None
  _SERVERGROUPARGS.fields_by_name['options']._serialized_options = b'\222M\002\010\001'
  _SERVERGROUPARGS.fields_by_name['autoscaling_config']._options = None
  _SERVERGROUPARGS.fields_by_name['autoscaling_config']._serialized_options = b'\222M\002\010\001'
  _SERVERGROUPARGS.fields_by_name['provisioned_scaling_config']._options = None
  _SERVERGROUPARGS.fields_by_name['provisioned_scaling_config']._serialized_options = b'\222M\002\010\001'
  _SERVERGROUPARGS._serialized_start=299
  _SERVERGROUPARGS._serialized_end=1207
  _SERVERGROUPARGS_OPTIONSENTRY._serialized_start=1094
  _SERVERGROUPARGS_OPTIONSENTRY._serialized_end=1152
  _TRANSFORMSERVERGROUPARGS._serialized_start=1209
  _TRANSFORMSERVERGROUPARGS._serialized_end=1269
  _FEATURESERVERGROUPARGS._serialized_start=1271
  _FEATURESERVERGROUPARGS._serialized_end=1295
  _SERVERGROUPREFERENCE._serialized_start=1297
  _SERVERGROUPREFERENCE._serialized_end=1404
# @@protoc_insertion_point(module_scope)
