# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/resource_provider__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.args import user_defined_function__client_pb2 as tecton__proto_dot_args_dot_user__defined__function____client__pb2
from tecton_proto.common import id__client_pb2 as tecton__proto_dot_common_dot_id____client__pb2
from tecton_proto.common import secret__client_pb2 as tecton__proto_dot_common_dot_secret____client__pb2
from tecton_proto.data import fco_metadata__client_pb2 as tecton__proto_dot_data_dot_fco__metadata____client__pb2
from tecton_proto.validation import validator__client_pb2 as tecton__proto_dot_validation_dot_validator____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n1tecton_proto/data/resource_provider__client.proto\x12\x11tecton_proto.data\x1a\x35tecton_proto/args/user_defined_function__client.proto\x1a$tecton_proto/common/id__client.proto\x1a(tecton_proto/common/secret__client.proto\x1a,tecton_proto/data/fco_metadata__client.proto\x1a/tecton_proto/validation/validator__client.proto\"\xea\x03\n\x10ResourceProvider\x12I\n\x14resource_provider_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x12resourceProviderId\x12\x41\n\x0c\x66\x63o_metadata\x18\x07 \x01(\x0b\x32\x1e.tecton_proto.data.FcoMetadataR\x0b\x66\x63oMetadata\x12J\n\x07secrets\x18\x03 \x03(\x0b\x32\x30.tecton_proto.data.ResourceProvider.SecretsEntryR\x07secrets\x12\x42\n\x08\x66unction\x18\x04 \x01(\x0b\x32&.tecton_proto.args.UserDefinedFunctionR\x08\x66unction\x12V\n\x0fvalidation_args\x18\t \x01(\x0b\x32-.tecton_proto.validation.ResourceProviderArgsR\x0evalidationArgs\x1a`\n\x0cSecretsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12:\n\x05value\x18\x02 \x01(\x0b\x32$.tecton_proto.common.SecretReferenceR\x05value:\x02\x38\x01\x42;\n\x0f\x63om.tecton.dataP\x01Z&github.com/tecton-ai/tecton_proto/data')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.data.resource_provider__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.dataP\001Z&github.com/tecton-ai/tecton_proto/data'
  _RESOURCEPROVIDER_SECRETSENTRY._options = None
  _RESOURCEPROVIDER_SECRETSENTRY._serialized_options = b'8\001'
  _RESOURCEPROVIDER._serialized_start=303
  _RESOURCEPROVIDER._serialized_end=793
  _RESOURCEPROVIDER_SECRETSENTRY._serialized_start=697
  _RESOURCEPROVIDER_SECRETSENTRY._serialized_end=793
# @@protoc_insertion_point(module_scope)
