# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/modelartifactservice/model_artifact_data__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.auth import principal__client_pb2 as tecton__proto_dot_auth_dot_principal____client__pb2
from tecton_proto.common import id__client_pb2 as tecton__proto_dot_common_dot_id____client__pb2
from tecton_proto.common import schema__client_pb2 as tecton__proto_dot_common_dot_schema____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nCtecton_proto/modelartifactservice/model_artifact_data__client.proto\x12!tecton_proto.modelartifactservice\x1a\x1fgoogle/protobuf/timestamp.proto\x1a)tecton_proto/auth/principal__client.proto\x1a$tecton_proto/common/id__client.proto\x1a(tecton_proto/common/schema__client.proto\"\x8c\t\n\x11ModelArtifactInfo\x12\'\n\x02id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x02id\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name\x12@\n\x04type\x18\x03 \x01(\x0e\x32,.tecton_proto.modelartifactservice.ModelTypeR\x04type\x12 \n\x0b\x64\x65scription\x18\x04 \x01(\tR\x0b\x64\x65scription\x12\x65\n\x0b\x66ile_hashes\x18\x05 \x03(\x0b\x32\x44.tecton_proto.modelartifactservice.ModelArtifactInfo.FileHashesEntryR\nfileHashes\x12>\n\x0cinput_schema\x18\x06 \x01(\x0b\x32\x1b.tecton_proto.common.SchemaR\x0binputSchema\x12@\n\routput_schema\x18\x07 \x01(\x0b\x32\x1b.tecton_proto.common.SchemaR\x0coutputSchema\x12R\n\x04tags\x18\t \x03(\x0b\x32>.tecton_proto.modelartifactservice.ModelArtifactInfo.TagsEntryR\x04tags\x12!\n\x0cstorage_path\x18\x08 \x01(\tR\x0bstoragePath\x12N\n\x06status\x18\n \x01(\x0e\x32\x36.tecton_proto.modelartifactservice.ModelArtifactStatusR\x06status\x12\x39\n\ncreated_at\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x39\n\nupdated_at\x18\x0c \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tupdatedAt\x12&\n\x0fmodel_file_path\x18\r \x01(\tR\rmodelFilePath\x12\x33\n\x16model_config_file_path\x18\x11 \x01(\tR\x13modelConfigFilePath\x12%\n\x0e\x61rtifact_files\x18\x12 \x03(\tR\rartifactFiles\x12\"\n\x0c\x65nvironments\x18\x0e \x03(\tR\x0c\x65nvironments\x12;\n\ncreated_by\x18\x0f \x01(\x0b\x32\x1c.tecton_proto.auth.PrincipalR\tcreatedBy\x12S\n\x14\x63reated_by_principal\x18\x10 \x01(\x0b\x32!.tecton_proto.auth.PrincipalBasicR\x12\x63reatedByPrincipal\x1a=\n\x0f\x46ileHashesEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x1a\x37\n\tTagsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"\xa2\x01\n\tModelInfo\x12\'\n\x02id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x02id\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name\x12(\n\x10model_public_uri\x18\x03 \x01(\tR\x0emodelPublicUri\x12.\n\x13metadata_public_uri\x18\x04 \x01(\tR\x11metadataPublicUri*t\n\x13ModelArtifactStatus\x12\x16\n\x12STATUS_UNSPECIFIED\x10\x00\x12\x10\n\x0cPENDING_FILE\x10\x01\x12\x10\n\x0cPENDING_SCAN\x10\x02\x12\t\n\x05READY\x10\x03\x12\t\n\x05\x45RROR\x10\x04\x12\x0b\n\x07\x44\x45LETED\x10\x05*[\n\tModelType\x12\x1a\n\x16MODEL_TYPE_UNSPECIFIED\x10\x00\x12\x19\n\x15TECTON_TEXT_EMBEDDING\x10\x01\x12\x0b\n\x07PYTORCH\x10\x02\x12\n\n\x06PYTHON\x10\x03\x42u\n\x1f\x63om.tecton.modelartifactserviceB\x18ModelArtifactServiceDataP\x01Z6github.com/tecton-ai/tecton_proto/modelartifactservice')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.modelartifactservice.model_artifact_data__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\037com.tecton.modelartifactserviceB\030ModelArtifactServiceDataP\001Z6github.com/tecton-ai/tecton_proto/modelartifactservice'
  _MODELARTIFACTINFO_FILEHASHESENTRY._options = None
  _MODELARTIFACTINFO_FILEHASHESENTRY._serialized_options = b'8\001'
  _MODELARTIFACTINFO_TAGSENTRY._options = None
  _MODELARTIFACTINFO_TAGSENTRY._serialized_options = b'8\001'
  _MODELARTIFACTSTATUS._serialized_start=1594
  _MODELARTIFACTSTATUS._serialized_end=1710
  _MODELTYPE._serialized_start=1712
  _MODELTYPE._serialized_end=1803
  _MODELARTIFACTINFO._serialized_start=263
  _MODELARTIFACTINFO._serialized_end=1427
  _MODELARTIFACTINFO_FILEHASHESENTRY._serialized_start=1309
  _MODELARTIFACTINFO_FILEHASHESENTRY._serialized_end=1370
  _MODELARTIFACTINFO_TAGSENTRY._serialized_start=1372
  _MODELARTIFACTINFO_TAGSENTRY._serialized_end=1427
  _MODELINFO._serialized_start=1430
  _MODELINFO._serialized_end=1592
# @@protoc_insertion_point(module_scope)
