# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/args/transformation__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.args import basic_info__client_pb2 as tecton__proto_dot_args_dot_basic__info____client__pb2
from tecton_proto.args import diff_options__client_pb2 as tecton__proto_dot_args_dot_diff__options____client__pb2
from tecton_proto.args import user_defined_function__client_pb2 as tecton__proto_dot_args_dot_user__defined__function____client__pb2
from tecton_proto.common import framework_version__client_pb2 as tecton__proto_dot_common_dot_framework__version____client__pb2
from tecton_proto.common import id__client_pb2 as tecton__proto_dot_common_dot_id____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n.tecton_proto/args/transformation__client.proto\x12\x11tecton_proto.args\x1a*tecton_proto/args/basic_info__client.proto\x1a,tecton_proto/args/diff_options__client.proto\x1a\x35tecton_proto/args/user_defined_function__client.proto\x1a\x33tecton_proto/common/framework_version__client.proto\x1a$tecton_proto/common/id__client.proto\"\x93\x05\n\x12TransformationArgs\x12\x44\n\x11transformation_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x10transformationId\x12\x37\n\x04info\x18\x02 \x01(\x0b\x32\x1c.tecton_proto.args.BasicInfoB\x05\x92M\x02\x10\x01R\x04info\x12\x46\n\x07version\x18\x07 \x01(\x0e\x32%.tecton_proto.common.FrameworkVersionB\x05\x92M\x02\x08\x05R\x07version\x12.\n\x0fprevent_destroy\x18\x08 \x01(\x08\x42\x05\x92M\x02\x08\x01R\x0epreventDestroy\x12S\n\x07options\x18\t \x03(\x0b\x32\x32.tecton_proto.args.TransformationArgs.OptionsEntryB\x05\x92M\x02\x08\x01R\x07options\x12]\n\x13transformation_mode\x18\x03 \x01(\x0e\x32%.tecton_proto.args.TransformationModeB\x05\x92M\x02\x08\x06R\x12transformationMode\x12R\n\ruser_function\x18\x04 \x01(\x0b\x32&.tecton_proto.args.UserDefinedFunctionB\x05\x92M\x02\x08\x06R\x0cuserFunction\x12#\n\tdocstring\x18\x06 \x01(\tB\x05\x92M\x02\x08\x01R\tdocstring\x12\x1d\n\nis_builtin\x18\x05 \x01(\x08R\tisBuiltin\x1a:\n\x0cOptionsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01*\xdd\x02\n\x12TransformationMode\x12#\n\x1fTRANSFORMATION_MODE_UNSPECIFIED\x10\x00\x12\x1f\n\x1bTRANSFORMATION_MODE_PYSPARK\x10\x01\x12!\n\x1dTRANSFORMATION_MODE_SPARK_SQL\x10\x02\x12\x1e\n\x1aTRANSFORMATION_MODE_PANDAS\x10\x03\x12%\n!TRANSFORMATION_MODE_SNOWFLAKE_SQL\x10\x04\x12\x1e\n\x1aTRANSFORMATION_MODE_PYTHON\x10\x05\x12 \n\x1cTRANSFORMATION_MODE_SNOWPARK\x10\x06\x12$\n TRANSFORMATION_MODE_BIGQUERY_SQL\x10\x08\x12)\n%TRANSFORMATION_MODE_NO_TRANSFORMATION\x10\t\"\x04\x08\x07\x10\x07\x42;\n\x0f\x63om.tecton.argsP\x01Z&github.com/tecton-ai/tecton_proto/args')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.args.transformation__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.argsP\001Z&github.com/tecton-ai/tecton_proto/args'
  _TRANSFORMATIONARGS_OPTIONSENTRY._options = None
  _TRANSFORMATIONARGS_OPTIONSENTRY._serialized_options = b'8\001'
  _TRANSFORMATIONARGS.fields_by_name['info']._options = None
  _TRANSFORMATIONARGS.fields_by_name['info']._serialized_options = b'\222M\002\020\001'
  _TRANSFORMATIONARGS.fields_by_name['version']._options = None
  _TRANSFORMATIONARGS.fields_by_name['version']._serialized_options = b'\222M\002\010\005'
  _TRANSFORMATIONARGS.fields_by_name['prevent_destroy']._options = None
  _TRANSFORMATIONARGS.fields_by_name['prevent_destroy']._serialized_options = b'\222M\002\010\001'
  _TRANSFORMATIONARGS.fields_by_name['options']._options = None
  _TRANSFORMATIONARGS.fields_by_name['options']._serialized_options = b'\222M\002\010\001'
  _TRANSFORMATIONARGS.fields_by_name['transformation_mode']._options = None
  _TRANSFORMATIONARGS.fields_by_name['transformation_mode']._serialized_options = b'\222M\002\010\006'
  _TRANSFORMATIONARGS.fields_by_name['user_function']._options = None
  _TRANSFORMATIONARGS.fields_by_name['user_function']._serialized_options = b'\222M\002\010\006'
  _TRANSFORMATIONARGS.fields_by_name['docstring']._options = None
  _TRANSFORMATIONARGS.fields_by_name['docstring']._serialized_options = b'\222M\002\010\001'
  _TRANSFORMATIONMODE._serialized_start=968
  _TRANSFORMATIONMODE._serialized_end=1317
  _TRANSFORMATIONARGS._serialized_start=306
  _TRANSFORMATIONARGS._serialized_end=965
  _TRANSFORMATIONARGS_OPTIONSENTRY._serialized_start=907
  _TRANSFORMATIONARGS_OPTIONSENTRY._serialized_end=965
# @@protoc_insertion_point(module_scope)
