from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

CLUSTER_DEFAULT: PythonVersion
DESCRIPTOR: _descriptor.FileDescriptor
PYTHON_3_9_0: PythonVersion
PYTHON_3_9_13: PythonVersion

class PythonVersion(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
