# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/serviceaccounts/service_accounts_service__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.auditlog import metadata__client_pb2 as tecton__proto_dot_auditlog_dot_metadata____client__pb2
from tecton_proto.auth import service__client_pb2 as tecton__proto_dot_auth_dot_service____client__pb2
from tecton_proto.data import service_account__client_pb2 as tecton__proto_dot_data_dot_service__account____client__pb2
from tecton_proto.metadataservice import metadata_service__client_pb2 as tecton__proto_dot_metadataservice_dot_metadata__service____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nCtecton_proto/serviceaccounts/service_accounts_service__client.proto\x12\x1ctecton_proto.serviceaccounts\x1a,tecton_proto/auditlog/metadata__client.proto\x1a\'tecton_proto/auth/service__client.proto\x1a/tecton_proto/data/service_account__client.proto\x1a;tecton_proto/metadataservice/metadata_service__client.proto\"<\n!CreateServiceAccountSecretRequest\x12\x17\n\x02id\x18\x01 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x02id\"`\n\"CreateServiceAccountSecretResponse\x12:\n\x06secret\x18\x01 \x01(\x0b\x32\".tecton_proto.data.NewClientSecretR\x06secret\"2\n ListServiceAccountSecretsRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\"q\n!ListServiceAccountSecretsResponse\x12L\n\x0e\x63lient_secrets\x18\x01 \x03(\x0b\x32%.tecton_proto.data.MaskedClientSecretR\rclientSecrets\"\x82\x01\n#ActivateServiceAccountSecretRequest\x12\x35\n\x12service_account_id\x18\x01 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x10serviceAccountId\x12$\n\tsecret_id\x18\x02 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x08secretId\"e\n$ActivateServiceAccountSecretResponse\x12=\n\x06secret\x18\x01 \x01(\x0b\x32%.tecton_proto.data.MaskedClientSecretR\x06secret\"\x84\x01\n%DeactivateServiceAccountSecretRequest\x12\x35\n\x12service_account_id\x18\x01 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x10serviceAccountId\x12$\n\tsecret_id\x18\x02 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x08secretId\"g\n&DeactivateServiceAccountSecretResponse\x12=\n\x06secret\x18\x01 \x01(\x0b\x32%.tecton_proto.data.MaskedClientSecretR\x06secret\"\x80\x01\n!DeleteServiceAccountSecretRequest\x12\x35\n\x12service_account_id\x18\x01 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x10serviceAccountId\x12$\n\tsecret_id\x18\x02 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x08secretId\"$\n\"DeleteServiceAccountSecretResponse2\xcd\x11\n\x16ServiceAccountsService\x12\xe9\x01\n\x1a\x43reateSingleServiceAccount\x12\x39.tecton_proto.metadataservice.CreateServiceAccountRequest\x1a:.tecton_proto.metadataservice.CreateServiceAccountResponse\"T\xa2\xbc\xe6\xc0\x05,**tecton/organization#create_service_account\xf2\x99\xbd\x46\x1d\x08\x01\x12\x16\x63reate_service_account\x1a\x01\x31\x12\x94\x01\n\x15SearchServiceAccounts\x12\x37.tecton_proto.metadataservice.GetServiceAccountsRequest\x1a\x38.tecton_proto.metadataservice.GetServiceAccountsResponse\"\x08\xa2\xbc\xe6\xc0\x05\x02@\x01\x12\xf4\x01\n\x1aUpdateSingleServiceAccount\x12\x39.tecton_proto.metadataservice.UpdateServiceAccountRequest\x1a:.tecton_proto.metadataservice.UpdateServiceAccountResponse\"_\xa2\xbc\xe6\xc0\x05\x37\x1a\x06\x08\x02\x12\x02id*-tecton/service_account#manage_service_account\xf2\x99\xbd\x46\x1d\x08\x01\x12\x16update_service_account\x1a\x01\x31\x12\xf4\x01\n\x1a\x44\x65leteSingleServiceAccount\x12\x39.tecton_proto.metadataservice.DeleteServiceAccountRequest\x1a:.tecton_proto.metadataservice.DeleteServiceAccountResponse\"_\xa2\xbc\xe6\xc0\x05\x37\x1a\x06\x08\x02\x12\x02id*-tecton/service_account#manage_service_account\xf2\x99\xbd\x46\x1d\x08\x01\x12\x16\x64\x65lete_service_account\x1a\x01\x31\x12\x85\x02\n\x1a\x43reateServiceAccountSecret\x12?.tecton_proto.serviceaccounts.CreateServiceAccountSecretRequest\x1a@.tecton_proto.serviceaccounts.CreateServiceAccountSecretResponse\"d\xa2\xbc\xe6\xc0\x05\x35\x1a\x06\x08\x02\x12\x02id*+tecton/service_account#manage_client_secret\xf2\x99\xbd\x46$\x08\x01\x12\x1d\x63reate_service_account_secret\x1a\x01\x31\x12\xd9\x01\n\x19ListServiceAccountSecrets\x12>.tecton_proto.serviceaccounts.ListServiceAccountSecretsRequest\x1a?.tecton_proto.serviceaccounts.ListServiceAccountSecretsResponse\";\xa2\xbc\xe6\xc0\x05\x35\x1a\x06\x08\x02\x12\x02id*+tecton/service_account#manage_client_secret\x12\x9d\x02\n\x1c\x41\x63tivateServiceAccountSecret\x12\x41.tecton_proto.serviceaccounts.ActivateServiceAccountSecretRequest\x1a\x42.tecton_proto.serviceaccounts.ActivateServiceAccountSecretResponse\"v\xa2\xbc\xe6\xc0\x05\x45\x1a\x16\x08\x02\x12\x12service_account_id*+tecton/service_account#manage_client_secret\xf2\x99\xbd\x46&\x08\x01\x12\x1f\x61\x63tivate_service_account_secret\x1a\x01\x31\x12\xa5\x02\n\x1e\x44\x65\x61\x63tivateServiceAccountSecret\x12\x43.tecton_proto.serviceaccounts.DeactivateServiceAccountSecretRequest\x1a\x44.tecton_proto.serviceaccounts.DeactivateServiceAccountSecretResponse\"x\xa2\xbc\xe6\xc0\x05\x45\x1a\x16\x08\x02\x12\x12service_account_id*+tecton/service_account#manage_client_secret\xf2\x99\xbd\x46(\x08\x01\x12!deactivate_service_account_secret\x1a\x01\x31\x12\x95\x02\n\x1a\x44\x65leteServiceAccountSecret\x12?.tecton_proto.serviceaccounts.DeleteServiceAccountSecretRequest\x1a@.tecton_proto.serviceaccounts.DeleteServiceAccountSecretResponse\"t\xa2\xbc\xe6\xc0\x05\x45\x1a\x16\x08\x02\x12\x12service_account_id*+tecton/service_account#manage_client_secret\xf2\x99\xbd\x46$\x08\x01\x12\x1d\x64\x65lete_service_account_secret\x1a\x01\x31\x42n\n\x1a\x63om.tecton.serviceaccountsB\x1bServiceAccountsServiceProtoP\x01Z1github.com/tecton-ai/tecton_proto/serviceaccounts')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.serviceaccounts.service_accounts_service__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\032com.tecton.serviceaccountsB\033ServiceAccountsServiceProtoP\001Z1github.com/tecton-ai/tecton_proto/serviceaccounts'
  _CREATESERVICEACCOUNTSECRETREQUEST.fields_by_name['id']._options = None
  _CREATESERVICEACCOUNTSECRETREQUEST.fields_by_name['id']._serialized_options = b'\352\231\275F\002\010\002'
  _ACTIVATESERVICEACCOUNTSECRETREQUEST.fields_by_name['service_account_id']._options = None
  _ACTIVATESERVICEACCOUNTSECRETREQUEST.fields_by_name['service_account_id']._serialized_options = b'\352\231\275F\002\010\002'
  _ACTIVATESERVICEACCOUNTSECRETREQUEST.fields_by_name['secret_id']._options = None
  _ACTIVATESERVICEACCOUNTSECRETREQUEST.fields_by_name['secret_id']._serialized_options = b'\352\231\275F\002\010\002'
  _DEACTIVATESERVICEACCOUNTSECRETREQUEST.fields_by_name['service_account_id']._options = None
  _DEACTIVATESERVICEACCOUNTSECRETREQUEST.fields_by_name['service_account_id']._serialized_options = b'\352\231\275F\002\010\002'
  _DEACTIVATESERVICEACCOUNTSECRETREQUEST.fields_by_name['secret_id']._options = None
  _DEACTIVATESERVICEACCOUNTSECRETREQUEST.fields_by_name['secret_id']._serialized_options = b'\352\231\275F\002\010\002'
  _DELETESERVICEACCOUNTSECRETREQUEST.fields_by_name['service_account_id']._options = None
  _DELETESERVICEACCOUNTSECRETREQUEST.fields_by_name['service_account_id']._serialized_options = b'\352\231\275F\002\010\002'
  _DELETESERVICEACCOUNTSECRETREQUEST.fields_by_name['secret_id']._options = None
  _DELETESERVICEACCOUNTSECRETREQUEST.fields_by_name['secret_id']._serialized_options = b'\352\231\275F\002\010\002'
  _SERVICEACCOUNTSSERVICE.methods_by_name['CreateSingleServiceAccount']._options = None
  _SERVICEACCOUNTSSERVICE.methods_by_name['CreateSingleServiceAccount']._serialized_options = b'\242\274\346\300\005,**tecton/organization#create_service_account\362\231\275F\035\010\001\022\026create_service_account\032\0011'
  _SERVICEACCOUNTSSERVICE.methods_by_name['SearchServiceAccounts']._options = None
  _SERVICEACCOUNTSSERVICE.methods_by_name['SearchServiceAccounts']._serialized_options = b'\242\274\346\300\005\002@\001'
  _SERVICEACCOUNTSSERVICE.methods_by_name['UpdateSingleServiceAccount']._options = None
  _SERVICEACCOUNTSSERVICE.methods_by_name['UpdateSingleServiceAccount']._serialized_options = b'\242\274\346\300\0057\032\006\010\002\022\002id*-tecton/service_account#manage_service_account\362\231\275F\035\010\001\022\026update_service_account\032\0011'
  _SERVICEACCOUNTSSERVICE.methods_by_name['DeleteSingleServiceAccount']._options = None
  _SERVICEACCOUNTSSERVICE.methods_by_name['DeleteSingleServiceAccount']._serialized_options = b'\242\274\346\300\0057\032\006\010\002\022\002id*-tecton/service_account#manage_service_account\362\231\275F\035\010\001\022\026delete_service_account\032\0011'
  _SERVICEACCOUNTSSERVICE.methods_by_name['CreateServiceAccountSecret']._options = None
  _SERVICEACCOUNTSSERVICE.methods_by_name['CreateServiceAccountSecret']._serialized_options = b'\242\274\346\300\0055\032\006\010\002\022\002id*+tecton/service_account#manage_client_secret\362\231\275F$\010\001\022\035create_service_account_secret\032\0011'
  _SERVICEACCOUNTSSERVICE.methods_by_name['ListServiceAccountSecrets']._options = None
  _SERVICEACCOUNTSSERVICE.methods_by_name['ListServiceAccountSecrets']._serialized_options = b'\242\274\346\300\0055\032\006\010\002\022\002id*+tecton/service_account#manage_client_secret'
  _SERVICEACCOUNTSSERVICE.methods_by_name['ActivateServiceAccountSecret']._options = None
  _SERVICEACCOUNTSSERVICE.methods_by_name['ActivateServiceAccountSecret']._serialized_options = b'\242\274\346\300\005E\032\026\010\002\022\022service_account_id*+tecton/service_account#manage_client_secret\362\231\275F&\010\001\022\037activate_service_account_secret\032\0011'
  _SERVICEACCOUNTSSERVICE.methods_by_name['DeactivateServiceAccountSecret']._options = None
  _SERVICEACCOUNTSSERVICE.methods_by_name['DeactivateServiceAccountSecret']._serialized_options = b'\242\274\346\300\005E\032\026\010\002\022\022service_account_id*+tecton/service_account#manage_client_secret\362\231\275F(\010\001\022!deactivate_service_account_secret\032\0011'
  _SERVICEACCOUNTSSERVICE.methods_by_name['DeleteServiceAccountSecret']._options = None
  _SERVICEACCOUNTSSERVICE.methods_by_name['DeleteServiceAccountSecret']._serialized_options = b'\242\274\346\300\005E\032\026\010\002\022\022service_account_id*+tecton/service_account#manage_client_secret\362\231\275F$\010\001\022\035delete_service_account_secret\032\0011'
  _CREATESERVICEACCOUNTSECRETREQUEST._serialized_start=298
  _CREATESERVICEACCOUNTSECRETREQUEST._serialized_end=358
  _CREATESERVICEACCOUNTSECRETRESPONSE._serialized_start=360
  _CREATESERVICEACCOUNTSECRETRESPONSE._serialized_end=456
  _LISTSERVICEACCOUNTSECRETSREQUEST._serialized_start=458
  _LISTSERVICEACCOUNTSECRETSREQUEST._serialized_end=508
  _LISTSERVICEACCOUNTSECRETSRESPONSE._serialized_start=510
  _LISTSERVICEACCOUNTSECRETSRESPONSE._serialized_end=623
  _ACTIVATESERVICEACCOUNTSECRETREQUEST._serialized_start=626
  _ACTIVATESERVICEACCOUNTSECRETREQUEST._serialized_end=756
  _ACTIVATESERVICEACCOUNTSECRETRESPONSE._serialized_start=758
  _ACTIVATESERVICEACCOUNTSECRETRESPONSE._serialized_end=859
  _DEACTIVATESERVICEACCOUNTSECRETREQUEST._serialized_start=862
  _DEACTIVATESERVICEACCOUNTSECRETREQUEST._serialized_end=994
  _DEACTIVATESERVICEACCOUNTSECRETRESPONSE._serialized_start=996
  _DEACTIVATESERVICEACCOUNTSECRETRESPONSE._serialized_end=1099
  _DELETESERVICEACCOUNTSECRETREQUEST._serialized_start=1102
  _DELETESERVICEACCOUNTSECRETREQUEST._serialized_end=1230
  _DELETESERVICEACCOUNTSECRETRESPONSE._serialized_start=1232
  _DELETESERVICEACCOUNTSECRETRESPONSE._serialized_end=1268
  _SERVICEACCOUNTSSERVICE._serialized_start=1271
  _SERVICEACCOUNTSSERVICE._serialized_end=3524
# @@protoc_insertion_point(module_scope)
