# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/principal/principal_service__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.auditlog import metadata__client_pb2 as tecton__proto_dot_auditlog_dot_metadata____client__pb2
from tecton_proto.auth import principal__client_pb2 as tecton__proto_dot_auth_dot_principal____client__pb2
from tecton_proto.auth import resource_role_assignments__client_pb2 as tecton__proto_dot_auth_dot_resource__role__assignments____client__pb2
from tecton_proto.auth import service__client_pb2 as tecton__proto_dot_auth_dot_service____client__pb2
from tecton_proto.data import principal_group__client_pb2 as tecton__proto_dot_data_dot_principal__group____client__pb2
from tecton_proto.data import user__client_pb2 as tecton__proto_dot_data_dot_user____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n6tecton_proto/principal/principal_service__client.proto\x12\x16tecton_proto.principal\x1a\x1fgoogle/protobuf/timestamp.proto\x1a,tecton_proto/auditlog/metadata__client.proto\x1a)tecton_proto/auth/principal__client.proto\x1a\x39tecton_proto/auth/resource_role_assignments__client.proto\x1a\'tecton_proto/auth/service__client.proto\x1a/tecton_proto/data/principal_group__client.proto\x1a$tecton_proto/data/user__client.proto\"\xa0\x01\n\x1b\x43reatePrincipalGroupRequest\x12\x1b\n\x04name\x18\x02 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x04name\x12)\n\x0b\x64\x65scription\x18\x03 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x0b\x64\x65scription\x12\x33\n\x11idp_mapping_names\x18\x04 \x03(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x0fidpMappingNamesJ\x04\x08\x01\x10\x02\"s\n\x1c\x43reatePrincipalGroupResponse\x12S\n\x0fprincipal_group\x18\x01 \x01(\x0b\x32!.tecton_proto.data.PrincipalGroupB\x07\xea\x99\xbd\x46\x02\x08\x02R\x0eprincipalGroup\"\xb3\x01\n\x1bUpdatePrincipalGroupRequest\x12\x17\n\x02id\x18\x01 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x02id\x12\x1b\n\x04name\x18\x02 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x04name\x12)\n\x0b\x64\x65scription\x18\x03 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x0b\x64\x65scription\x12\x33\n\x11idp_mapping_names\x18\x04 \x03(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x0fidpMappingNames\"s\n\x1cUpdatePrincipalGroupResponse\x12S\n\x0fprincipal_group\x18\x01 \x01(\x0b\x32!.tecton_proto.data.PrincipalGroupB\x07\xea\x99\xbd\x46\x02\x08\x02R\x0eprincipalGroup\"\x12\n\x10ListUsersRequest\"B\n\x11ListUsersResponse\x12-\n\x05users\x18\x01 \x03(\x0b\x32\x17.tecton_proto.data.UserR\x05users\"9\n\x1c\x44\x65letePrincipalGroupsRequest\x12\x19\n\x03ids\x18\x01 \x03(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x03ids\"\x1f\n\x1d\x44\x65letePrincipalGroupsResponse\"N\n\"ListPrincipalGroupsDetailedRequest\x12\x16\n\x06search\x18\x01 \x01(\tR\x06search\x12\x10\n\x03ids\x18\x02 \x03(\tR\x03ids\"m\n#ListPrincipalGroupsDetailedResponse\x12\x46\n\x06groups\x18\x01 \x03(\x0b\x32..tecton_proto.principal.PrincipalGroupDetailedR\x06groups\"\xd8\x03\n\x16PrincipalGroupDetailed\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name\x12 \n\x0b\x64\x65scription\x18\x03 \x01(\tR\x0b\x64\x65scription\x12*\n\x11idp_mapping_names\x18\x04 \x03(\tR\x0fidpMappingNames\x12\x34\n\x16is_membership_editable\x18\x05 \x01(\x08R\x14isMembershipEditable\x12\x39\n\ncreated_at\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12!\n\x0c\x61\x63\x63ount_type\x18\x07 \x01(\tR\x0b\x61\x63\x63ountType\x12\x1f\n\x0bnum_members\x18\x08 \x01(\x05R\nnumMembers\x12O\n\nworkspaces\x18\n \x03(\x0b\x32/.tecton_proto.auth.ResourceAndRoleAssignmentsV2R\nworkspaces\x12@\n\ncreated_by\x18\x0b \x01(\x0b\x32!.tecton_proto.auth.PrincipalBasicR\tcreatedByJ\x04\x08\t\x10\n\"\xac\x01\n\x1f\x41\x64\x64PrincipalGroupMembersRequest\x12\x35\n\x12principal_group_id\x18\x01 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x10principalGroupId\x12R\n\x07members\x18\x02 \x03(\x0b\x32/.tecton_proto.principal.AddPrincipalGroupMemberB\x07\xea\x99\xbd\x46\x02\x08\x02R\x07members\"\x97\x01\n\x17\x41\x64\x64PrincipalGroupMember\x12P\n\x0eprincipal_type\x18\x01 \x01(\x0e\x32 .tecton_proto.auth.PrincipalTypeB\x07\xea\x99\xbd\x46\x02\x08\x02R\rprincipalType\x12*\n\x0cprincipal_id\x18\x02 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x0bprincipalId\"\"\n AddPrincipalGroupMembersResponse\"\x95\x01\n ListPrincipalGroupMembersRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x16\n\x06search\x18\x02 \x01(\tR\x06search\x12I\n\x0fprincipal_types\x18\x03 \x03(\x0e\x32 .tecton_proto.auth.PrincipalTypeR\x0eprincipalTypes\"k\n!ListPrincipalGroupMembersResponse\x12\x46\n\x07members\x18\x01 \x03(\x0b\x32,.tecton_proto.principal.PrincipalGroupMemberR\x07members\"\x86\x01\n\x14PrincipalGroupMember\x12?\n\tprincipal\x18\x01 \x01(\x0b\x32!.tecton_proto.auth.PrincipalBasicR\tprincipal\x12-\n\x13is_from_idp_mapping\x18\x02 \x01(\x08R\x10isFromIdpMapping\"\x94\x01\n\"RemovePrincipalGroupMembersRequest\x12\x17\n\x02id\x18\x01 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x02id\x12U\n\x07members\x18\x02 \x03(\x0b\x32\x32.tecton_proto.principal.RemovePrincipalGroupMemberB\x07\xea\x99\xbd\x46\x02\x08\x02R\x07members\"\x9a\x01\n\x1aRemovePrincipalGroupMember\x12P\n\x0eprincipal_type\x18\x01 \x01(\x0e\x32 .tecton_proto.auth.PrincipalTypeB\x07\xea\x99\xbd\x46\x02\x08\x02R\rprincipalType\x12*\n\x0cprincipal_id\x18\x02 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x0bprincipalId\"%\n#RemovePrincipalGroupMembersResponse\"\xac\x01\n&ListPrincipalGroupsForPrincipalRequest\x12G\n\x0eprincipal_type\x18\x01 \x01(\x0e\x32 .tecton_proto.auth.PrincipalTypeR\rprincipalType\x12!\n\x0cprincipal_id\x18\x02 \x01(\tR\x0bprincipalId\x12\x16\n\x06search\x18\x03 \x01(\tR\x06search\"u\n\'ListPrincipalGroupsForPrincipalResponse\x12J\n\x06groups\x18\x01 \x03(\x0b\x32\x32.tecton_proto.principal.PrincipalGroupForPrincipalR\x06groups\"\x84\x01\n\x1aPrincipalGroupForPrincipal\x12\x37\n\x05group\x18\x01 \x01(\x0b\x32!.tecton_proto.data.PrincipalGroupR\x05group\x12-\n\x13is_from_idp_mapping\x18\x02 \x01(\x08R\x10isFromIdpMapping2\xa2\x0e\n\x10PrincipalService\x12\xd7\x01\n\x14\x43reatePrincipalGroup\x12\x33.tecton_proto.principal.CreatePrincipalGroupRequest\x1a\x34.tecton_proto.principal.CreatePrincipalGroupResponse\"T\xa2\xbc\xe6\xc0\x05,**tecton/organization#create_principal_group\xf2\x99\xbd\x46\x1d\x08\x01\x12\x16\x63reate_principal_group\x1a\x01\x31\x12\xd7\x01\n\x14UpdatePrincipalGroup\x12\x33.tecton_proto.principal.UpdatePrincipalGroupRequest\x1a\x34.tecton_proto.principal.UpdatePrincipalGroupResponse\"T\xa2\xbc\xe6\xc0\x05,**tecton/organization#manage_principal_group\xf2\x99\xbd\x46\x1d\x08\x01\x12\x16update_principal_group\x1a\x01\x31\x12\xdb\x01\n\x15\x44\x65letePrincipalGroups\x12\x34.tecton_proto.principal.DeletePrincipalGroupsRequest\x1a\x35.tecton_proto.principal.DeletePrincipalGroupsResponse\"U\xa2\xbc\xe6\xc0\x05,**tecton/organization#manage_principal_group\xf2\x99\xbd\x46\x1e\x08\x01\x12\x17\x64\x65lete_principal_groups\x1a\x01\x31\x12\xa0\x01\n\x1bListPrincipalGroupsDetailed\x12:.tecton_proto.principal.ListPrincipalGroupsDetailedRequest\x1a;.tecton_proto.principal.ListPrincipalGroupsDetailedResponse\"\x08\xa2\xbc\xe6\xc0\x05\x02@\x01\x12\xfb\x01\n\x18\x41\x64\x64PrincipalGroupMembers\x12\x37.tecton_proto.principal.AddPrincipalGroupMembersRequest\x1a\x38.tecton_proto.principal.AddPrincipalGroupMembersResponse\"l\xa2\xbc\xe6\xc0\x05?\x1a\x16\x08\x03\x12\x12principal_group_id*%tecton/principal_group#manage_members\xf2\x99\xbd\x46\"\x08\x01\x12\x1b\x61\x64\x64_principal_group_members\x1a\x01\x31\x12\xc5\x01\n\x19ListPrincipalGroupMembers\x12\x38.tecton_proto.principal.ListPrincipalGroupMembersRequest\x1a\x39.tecton_proto.principal.ListPrincipalGroupMembersResponse\"3\xa2\xbc\xe6\xc0\x05-\x1a\x06\x08\x03\x12\x02id*#tecton/principal_group#list_members\x12j\n\tListUsers\x12(.tecton_proto.principal.ListUsersRequest\x1a).tecton_proto.principal.ListUsersResponse\"\x08\xa2\xbc\xe6\xc0\x05\x02@\x01\x12\xf7\x01\n\x1bRemovePrincipalGroupMembers\x12:.tecton_proto.principal.RemovePrincipalGroupMembersRequest\x1a;.tecton_proto.principal.RemovePrincipalGroupMembersResponse\"_\xa2\xbc\xe6\xc0\x05/\x1a\x06\x08\x03\x12\x02id*%tecton/principal_group#manage_members\xf2\x99\xbd\x46%\x08\x01\x12\x1eremove_principal_group_members\x1a\x01\x31\x12\xac\x01\n\x1fListPrincipalGroupsForPrincipal\x12>.tecton_proto.principal.ListPrincipalGroupsForPrincipalRequest\x1a?.tecton_proto.principal.ListPrincipalGroupsForPrincipalResponse\"\x08\xa2\xbc\xe6\xc0\x05\x02@\x01\x42\\\n\x14\x63om.tecton.principalB\x15PrincipalServiceProtoP\x01Z+github.com/tecton-ai/tecton_proto/principal')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.principal.principal_service__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024com.tecton.principalB\025PrincipalServiceProtoP\001Z+github.com/tecton-ai/tecton_proto/principal'
  _CREATEPRINCIPALGROUPREQUEST.fields_by_name['name']._options = None
  _CREATEPRINCIPALGROUPREQUEST.fields_by_name['name']._serialized_options = b'\352\231\275F\002\010\002'
  _CREATEPRINCIPALGROUPREQUEST.fields_by_name['description']._options = None
  _CREATEPRINCIPALGROUPREQUEST.fields_by_name['description']._serialized_options = b'\352\231\275F\002\010\002'
  _CREATEPRINCIPALGROUPREQUEST.fields_by_name['idp_mapping_names']._options = None
  _CREATEPRINCIPALGROUPREQUEST.fields_by_name['idp_mapping_names']._serialized_options = b'\352\231\275F\002\010\002'
  _CREATEPRINCIPALGROUPRESPONSE.fields_by_name['principal_group']._options = None
  _CREATEPRINCIPALGROUPRESPONSE.fields_by_name['principal_group']._serialized_options = b'\352\231\275F\002\010\002'
  _UPDATEPRINCIPALGROUPREQUEST.fields_by_name['id']._options = None
  _UPDATEPRINCIPALGROUPREQUEST.fields_by_name['id']._serialized_options = b'\352\231\275F\002\010\002'
  _UPDATEPRINCIPALGROUPREQUEST.fields_by_name['name']._options = None
  _UPDATEPRINCIPALGROUPREQUEST.fields_by_name['name']._serialized_options = b'\352\231\275F\002\010\002'
  _UPDATEPRINCIPALGROUPREQUEST.fields_by_name['description']._options = None
  _UPDATEPRINCIPALGROUPREQUEST.fields_by_name['description']._serialized_options = b'\352\231\275F\002\010\002'
  _UPDATEPRINCIPALGROUPREQUEST.fields_by_name['idp_mapping_names']._options = None
  _UPDATEPRINCIPALGROUPREQUEST.fields_by_name['idp_mapping_names']._serialized_options = b'\352\231\275F\002\010\002'
  _UPDATEPRINCIPALGROUPRESPONSE.fields_by_name['principal_group']._options = None
  _UPDATEPRINCIPALGROUPRESPONSE.fields_by_name['principal_group']._serialized_options = b'\352\231\275F\002\010\002'
  _DELETEPRINCIPALGROUPSREQUEST.fields_by_name['ids']._options = None
  _DELETEPRINCIPALGROUPSREQUEST.fields_by_name['ids']._serialized_options = b'\352\231\275F\002\010\002'
  _ADDPRINCIPALGROUPMEMBERSREQUEST.fields_by_name['principal_group_id']._options = None
  _ADDPRINCIPALGROUPMEMBERSREQUEST.fields_by_name['principal_group_id']._serialized_options = b'\352\231\275F\002\010\002'
  _ADDPRINCIPALGROUPMEMBERSREQUEST.fields_by_name['members']._options = None
  _ADDPRINCIPALGROUPMEMBERSREQUEST.fields_by_name['members']._serialized_options = b'\352\231\275F\002\010\002'
  _ADDPRINCIPALGROUPMEMBER.fields_by_name['principal_type']._options = None
  _ADDPRINCIPALGROUPMEMBER.fields_by_name['principal_type']._serialized_options = b'\352\231\275F\002\010\002'
  _ADDPRINCIPALGROUPMEMBER.fields_by_name['principal_id']._options = None
  _ADDPRINCIPALGROUPMEMBER.fields_by_name['principal_id']._serialized_options = b'\352\231\275F\002\010\002'
  _REMOVEPRINCIPALGROUPMEMBERSREQUEST.fields_by_name['id']._options = None
  _REMOVEPRINCIPALGROUPMEMBERSREQUEST.fields_by_name['id']._serialized_options = b'\352\231\275F\002\010\002'
  _REMOVEPRINCIPALGROUPMEMBERSREQUEST.fields_by_name['members']._options = None
  _REMOVEPRINCIPALGROUPMEMBERSREQUEST.fields_by_name['members']._serialized_options = b'\352\231\275F\002\010\002'
  _REMOVEPRINCIPALGROUPMEMBER.fields_by_name['principal_type']._options = None
  _REMOVEPRINCIPALGROUPMEMBER.fields_by_name['principal_type']._serialized_options = b'\352\231\275F\002\010\002'
  _REMOVEPRINCIPALGROUPMEMBER.fields_by_name['principal_id']._options = None
  _REMOVEPRINCIPALGROUPMEMBER.fields_by_name['principal_id']._serialized_options = b'\352\231\275F\002\010\002'
  _PRINCIPALSERVICE.methods_by_name['CreatePrincipalGroup']._options = None
  _PRINCIPALSERVICE.methods_by_name['CreatePrincipalGroup']._serialized_options = b'\242\274\346\300\005,**tecton/organization#create_principal_group\362\231\275F\035\010\001\022\026create_principal_group\032\0011'
  _PRINCIPALSERVICE.methods_by_name['UpdatePrincipalGroup']._options = None
  _PRINCIPALSERVICE.methods_by_name['UpdatePrincipalGroup']._serialized_options = b'\242\274\346\300\005,**tecton/organization#manage_principal_group\362\231\275F\035\010\001\022\026update_principal_group\032\0011'
  _PRINCIPALSERVICE.methods_by_name['DeletePrincipalGroups']._options = None
  _PRINCIPALSERVICE.methods_by_name['DeletePrincipalGroups']._serialized_options = b'\242\274\346\300\005,**tecton/organization#manage_principal_group\362\231\275F\036\010\001\022\027delete_principal_groups\032\0011'
  _PRINCIPALSERVICE.methods_by_name['ListPrincipalGroupsDetailed']._options = None
  _PRINCIPALSERVICE.methods_by_name['ListPrincipalGroupsDetailed']._serialized_options = b'\242\274\346\300\005\002@\001'
  _PRINCIPALSERVICE.methods_by_name['AddPrincipalGroupMembers']._options = None
  _PRINCIPALSERVICE.methods_by_name['AddPrincipalGroupMembers']._serialized_options = b'\242\274\346\300\005?\032\026\010\003\022\022principal_group_id*%tecton/principal_group#manage_members\362\231\275F\"\010\001\022\033add_principal_group_members\032\0011'
  _PRINCIPALSERVICE.methods_by_name['ListPrincipalGroupMembers']._options = None
  _PRINCIPALSERVICE.methods_by_name['ListPrincipalGroupMembers']._serialized_options = b'\242\274\346\300\005-\032\006\010\003\022\002id*#tecton/principal_group#list_members'
  _PRINCIPALSERVICE.methods_by_name['ListUsers']._options = None
  _PRINCIPALSERVICE.methods_by_name['ListUsers']._serialized_options = b'\242\274\346\300\005\002@\001'
  _PRINCIPALSERVICE.methods_by_name['RemovePrincipalGroupMembers']._options = None
  _PRINCIPALSERVICE.methods_by_name['RemovePrincipalGroupMembers']._serialized_options = b'\242\274\346\300\005/\032\006\010\003\022\002id*%tecton/principal_group#manage_members\362\231\275F%\010\001\022\036remove_principal_group_members\032\0011'
  _PRINCIPALSERVICE.methods_by_name['ListPrincipalGroupsForPrincipal']._options = None
  _PRINCIPALSERVICE.methods_by_name['ListPrincipalGroupsForPrincipal']._serialized_options = b'\242\274\346\300\005\002@\001'
  _CREATEPRINCIPALGROUPREQUEST._serialized_start=392
  _CREATEPRINCIPALGROUPREQUEST._serialized_end=552
  _CREATEPRINCIPALGROUPRESPONSE._serialized_start=554
  _CREATEPRINCIPALGROUPRESPONSE._serialized_end=669
  _UPDATEPRINCIPALGROUPREQUEST._serialized_start=672
  _UPDATEPRINCIPALGROUPREQUEST._serialized_end=851
  _UPDATEPRINCIPALGROUPRESPONSE._serialized_start=853
  _UPDATEPRINCIPALGROUPRESPONSE._serialized_end=968
  _LISTUSERSREQUEST._serialized_start=970
  _LISTUSERSREQUEST._serialized_end=988
  _LISTUSERSRESPONSE._serialized_start=990
  _LISTUSERSRESPONSE._serialized_end=1056
  _DELETEPRINCIPALGROUPSREQUEST._serialized_start=1058
  _DELETEPRINCIPALGROUPSREQUEST._serialized_end=1115
  _DELETEPRINCIPALGROUPSRESPONSE._serialized_start=1117
  _DELETEPRINCIPALGROUPSRESPONSE._serialized_end=1148
  _LISTPRINCIPALGROUPSDETAILEDREQUEST._serialized_start=1150
  _LISTPRINCIPALGROUPSDETAILEDREQUEST._serialized_end=1228
  _LISTPRINCIPALGROUPSDETAILEDRESPONSE._serialized_start=1230
  _LISTPRINCIPALGROUPSDETAILEDRESPONSE._serialized_end=1339
  _PRINCIPALGROUPDETAILED._serialized_start=1342
  _PRINCIPALGROUPDETAILED._serialized_end=1814
  _ADDPRINCIPALGROUPMEMBERSREQUEST._serialized_start=1817
  _ADDPRINCIPALGROUPMEMBERSREQUEST._serialized_end=1989
  _ADDPRINCIPALGROUPMEMBER._serialized_start=1992
  _ADDPRINCIPALGROUPMEMBER._serialized_end=2143
  _ADDPRINCIPALGROUPMEMBERSRESPONSE._serialized_start=2145
  _ADDPRINCIPALGROUPMEMBERSRESPONSE._serialized_end=2179
  _LISTPRINCIPALGROUPMEMBERSREQUEST._serialized_start=2182
  _LISTPRINCIPALGROUPMEMBERSREQUEST._serialized_end=2331
  _LISTPRINCIPALGROUPMEMBERSRESPONSE._serialized_start=2333
  _LISTPRINCIPALGROUPMEMBERSRESPONSE._serialized_end=2440
  _PRINCIPALGROUPMEMBER._serialized_start=2443
  _PRINCIPALGROUPMEMBER._serialized_end=2577
  _REMOVEPRINCIPALGROUPMEMBERSREQUEST._serialized_start=2580
  _REMOVEPRINCIPALGROUPMEMBERSREQUEST._serialized_end=2728
  _REMOVEPRINCIPALGROUPMEMBER._serialized_start=2731
  _REMOVEPRINCIPALGROUPMEMBER._serialized_end=2885
  _REMOVEPRINCIPALGROUPMEMBERSRESPONSE._serialized_start=2887
  _REMOVEPRINCIPALGROUPMEMBERSRESPONSE._serialized_end=2924
  _LISTPRINCIPALGROUPSFORPRINCIPALREQUEST._serialized_start=2927
  _LISTPRINCIPALGROUPSFORPRINCIPALREQUEST._serialized_end=3099
  _LISTPRINCIPALGROUPSFORPRINCIPALRESPONSE._serialized_start=3101
  _LISTPRINCIPALGROUPSFORPRINCIPALRESPONSE._serialized_end=3218
  _PRINCIPALGROUPFORPRINCIPAL._serialized_start=3221
  _PRINCIPALGROUPFORPRINCIPAL._serialized_end=3353
  _PRINCIPALSERVICE._serialized_start=3356
  _PRINCIPALSERVICE._serialized_end=5182
# @@protoc_insertion_point(module_scope)
