# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/args/pipeline__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.args import diff_options__client_pb2 as tecton__proto_dot_args_dot_diff__options____client__pb2
from tecton_proto.args import feature_service__client_pb2 as tecton__proto_dot_args_dot_feature__service____client__pb2
from tecton_proto.common import id__client_pb2 as tecton__proto_dot_common_dot_id____client__pb2
from tecton_proto.common import schema__client_pb2 as tecton__proto_dot_common_dot_schema____client__pb2
from tecton_proto.common import spark_schema__client_pb2 as tecton__proto_dot_common_dot_spark__schema____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n(tecton_proto/args/pipeline__client.proto\x12\x11tecton_proto.args\x1a\x1egoogle/protobuf/duration.proto\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a,tecton_proto/args/diff_options__client.proto\x1a/tecton_proto/args/feature_service__client.proto\x1a$tecton_proto/common/id__client.proto\x1a(tecton_proto/common/schema__client.proto\x1a.tecton_proto/common/spark_schema__client.proto\"K\n\x08Pipeline\x12\x33\n\x04root\x18\x01 \x01(\x0b\x32\x1f.tecton_proto.args.PipelineNodeR\x04rootJ\x04\x08\x02\x10\x03J\x04\x08\x03\x10\x04\"\xd0\x05\n\x0cPipelineNode\x12X\n\x13transformation_node\x18\x01 \x01(\x0b\x32%.tecton_proto.args.TransformationNodeH\x00R\x12transformationNode\x12M\n\x10\x64\x61ta_source_node\x18\x02 \x01(\x0b\x32!.tecton_proto.args.DataSourceNodeH\x00R\x0e\x64\x61taSourceNode\x12\x46\n\rconstant_node\x18\x03 \x01(\x0b\x32\x1f.tecton_proto.args.ConstantNodeH\x00R\x0c\x63onstantNode\x12\x63\n\x18request_data_source_node\x18\x04 \x01(\x0b\x32(.tecton_proto.args.RequestDataSourceNodeH\x00R\x15requestDataSourceNode\x12P\n\x11\x66\x65\x61ture_view_node\x18\x05 \x01(\x0b\x32\".tecton_proto.args.FeatureViewNodeH\x00R\x0f\x66\x65\x61tureViewNode\x12q\n\x1cmaterialization_context_node\x18\x07 \x01(\x0b\x32-.tecton_proto.args.MaterializationContextNodeH\x00R\x1amaterializationContextNode\x12\x43\n\x0c\x63ontext_node\x18\x08 \x01(\x0b\x32\x1e.tecton_proto.args.ContextNodeH\x00R\x0b\x63ontextNode\x12M\n\x10join_inputs_node\x18\t \x01(\x0b\x32!.tecton_proto.args.JoinInputsNodeH\x00R\x0ejoinInputsNodeB\x0b\n\tnode_typeJ\x04\x08\x06\x10\x07\"\x89\x01\n\x15RequestDataSourceNode\x12Q\n\x0frequest_context\x18\x01 \x01(\x0b\x32!.tecton_proto.args.RequestContextB\x05\x92M\x02 \x04R\x0erequestContext\x12\x1d\n\ninput_name\x18\x02 \x01(\tR\tinputName\"\x8c\x01\n\x0eRequestContext\x12@\n\rtecton_schema\x18\x02 \x01(\x0b\x32\x1b.tecton_proto.common.SchemaR\x0ctectonSchema\x12\x38\n\x06schema\x18\x01 \x01(\x0b\x32 .tecton_proto.common.SparkSchemaR\x06schema\"\xc3\x01\n\x0f\x46\x65\x61tureViewNode\x12?\n\x0f\x66\x65\x61ture_view_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\rfeatureViewId\x12P\n\x11\x66\x65\x61ture_reference\x18\x02 \x01(\x0b\x32#.tecton_proto.args.FeatureReferenceR\x10\x66\x65\x61tureReference\x12\x1d\n\ninput_name\x18\x03 \x01(\tR\tinputName\"\x8c\x01\n\x12TransformationNode\x12\x44\n\x11transformation_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x10transformationId\x12\x30\n\x06inputs\x18\x02 \x03(\x0b\x32\x18.tecton_proto.args.InputR\x06inputs\"G\n\x0eJoinInputsNode\x12\x35\n\x05nodes\x18\x02 \x03(\x0b\x32\x1f.tecton_proto.args.PipelineNodeR\x05nodes\"\x84\x01\n\x05Input\x12\x1d\n\targ_index\x18\x01 \x01(\x05H\x00R\x08\x61rgIndex\x12\x1b\n\x08\x61rg_name\x18\x02 \x01(\tH\x00R\x07\x61rgName\x12\x33\n\x04node\x18\x03 \x01(\x0b\x32\x1f.tecton_proto.args.PipelineNodeR\x04nodeB\n\n\x08\x61rg_type\"\x81\x05\n\x0e\x44\x61taSourceNode\x12L\n\x16virtual_data_source_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x13virtualDataSourceId\x12\x33\n\x06window\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationH\x00R\x06window\x12\x45\n\x1awindow_unbounded_preceding\x18\x04 \x01(\x08\x42\x05\x92M\x02 \x12H\x00R\x18windowUnboundedPreceding\x12\x32\n\x10window_unbounded\x18\x06 \x01(\x08\x42\x05\x92M\x02 \x12H\x00R\x0fwindowUnbounded\x12N\n\x11start_time_offset\x18\x07 \x01(\x0b\x32\x19.google.protobuf.DurationB\x05\x92M\x02 \x12H\x00R\x0fstartTimeOffset\x12\x42\n\x0fschedule_offset\x18\x03 \x01(\x0b\x32\x19.google.protobuf.DurationR\x0escheduleOffset\x12\x1d\n\ninput_name\x18\x05 \x01(\tR\tinputName\x12T\n\x11\x66ilter_start_time\x18\x08 \x01(\x0b\x32!.tecton_proto.args.FilterDateTimeB\x05\x92M\x02 \x12R\x0f\x66ilterStartTime\x12P\n\x0f\x66ilter_end_time\x18\t \x01(\x0b\x32!.tecton_proto.args.FilterDateTimeB\x05\x92M\x02 \x12R\rfilterEndTimeB\x16\n\x14\x64\x61ta_window_override\"\xaa\x01\n\x0e\x46ilterDateTime\x12:\n\ttimestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00R\ttimestamp\x12\x46\n\rrelative_time\x18\x02 \x01(\x0b\x32\x1f.tecton_proto.args.RelativeTimeH\x00R\x0crelativeTimeB\x14\n\x12time_specification\"\x8a\x01\n\x0cRelativeTime\x12G\n\x0etime_reference\x18\x01 \x01(\x0e\x32 .tecton_proto.args.TimeReferenceR\rtimeReference\x12\x31\n\x06offset\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationR\x06offset\"\xe0\x01\n\x0c\x43onstantNode\x12#\n\x0cstring_const\x18\x01 \x01(\tH\x00R\x0bstringConst\x12\x1d\n\tint_const\x18\x02 \x01(\tH\x00R\x08intConst\x12!\n\x0b\x66loat_const\x18\x03 \x01(\tH\x00R\nfloatConst\x12\x1f\n\nbool_const\x18\x04 \x01(\x08H\x00R\tboolConst\x12\x37\n\nnull_const\x18\x05 \x01(\x0b\x32\x16.google.protobuf.EmptyH\x00R\tnullConstB\x0f\n\rconstant_type\"\x1c\n\x1aMaterializationContextNode\"o\n\x0b\x43ontextNode\x12\x41\n\x0c\x63ontext_type\x18\x01 \x01(\x0e\x32\x1e.tecton_proto.args.ContextTypeR\x0b\x63ontextType\x12\x1d\n\ninput_name\x18\x02 \x01(\tR\tinputName*\xd3\x01\n\rTimeReference\x12\x1e\n\x1aTIME_REFERENCE_UNSPECIFIED\x10\x00\x12-\n)TIME_REFERENCE_MATERIALIZATION_START_TIME\x10\x01\x12+\n\'TIME_REFERENCE_MATERIALIZATION_END_TIME\x10\x02\x12!\n\x1dTIME_REFERENCE_UNBOUNDED_PAST\x10\x03\x12#\n\x1fTIME_REFERENCE_UNBOUNDED_FUTURE\x10\x04*h\n\x0b\x43ontextType\x12\x1c\n\x18\x43ONTEXT_TYPE_UNSPECIFIED\x10\x00\x12 \n\x1c\x43ONTEXT_TYPE_MATERIALIZATION\x10\x01\x12\x19\n\x15\x43ONTEXT_TYPE_REALTIME\x10\x02\x42;\n\x0f\x63om.tecton.argsP\x01Z&github.com/tecton-ai/tecton_proto/args')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.args.pipeline__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.argsP\001Z&github.com/tecton-ai/tecton_proto/args'
  _REQUESTDATASOURCENODE.fields_by_name['request_context']._options = None
  _REQUESTDATASOURCENODE.fields_by_name['request_context']._serialized_options = b'\222M\002 \004'
  _DATASOURCENODE.fields_by_name['window_unbounded_preceding']._options = None
  _DATASOURCENODE.fields_by_name['window_unbounded_preceding']._serialized_options = b'\222M\002 \022'
  _DATASOURCENODE.fields_by_name['window_unbounded']._options = None
  _DATASOURCENODE.fields_by_name['window_unbounded']._serialized_options = b'\222M\002 \022'
  _DATASOURCENODE.fields_by_name['start_time_offset']._options = None
  _DATASOURCENODE.fields_by_name['start_time_offset']._serialized_options = b'\222M\002 \022'
  _DATASOURCENODE.fields_by_name['filter_start_time']._options = None
  _DATASOURCENODE.fields_by_name['filter_start_time']._serialized_options = b'\222M\002 \022'
  _DATASOURCENODE.fields_by_name['filter_end_time']._options = None
  _DATASOURCENODE.fields_by_name['filter_end_time']._serialized_options = b'\222M\002 \022'
  _TIMEREFERENCE._serialized_start=3341
  _TIMEREFERENCE._serialized_end=3552
  _CONTEXTTYPE._serialized_start=3554
  _CONTEXTTYPE._serialized_end=3658
  _PIPELINE._serialized_start=380
  _PIPELINE._serialized_end=455
  _PIPELINENODE._serialized_start=458
  _PIPELINENODE._serialized_end=1178
  _REQUESTDATASOURCENODE._serialized_start=1181
  _REQUESTDATASOURCENODE._serialized_end=1318
  _REQUESTCONTEXT._serialized_start=1321
  _REQUESTCONTEXT._serialized_end=1461
  _FEATUREVIEWNODE._serialized_start=1464
  _FEATUREVIEWNODE._serialized_end=1659
  _TRANSFORMATIONNODE._serialized_start=1662
  _TRANSFORMATIONNODE._serialized_end=1802
  _JOININPUTSNODE._serialized_start=1804
  _JOININPUTSNODE._serialized_end=1875
  _INPUT._serialized_start=1878
  _INPUT._serialized_end=2010
  _DATASOURCENODE._serialized_start=2013
  _DATASOURCENODE._serialized_end=2654
  _FILTERDATETIME._serialized_start=2657
  _FILTERDATETIME._serialized_end=2827
  _RELATIVETIME._serialized_start=2830
  _RELATIVETIME._serialized_end=2968
  _CONSTANTNODE._serialized_start=2971
  _CONSTANTNODE._serialized_end=3195
  _MATERIALIZATIONCONTEXTNODE._serialized_start=3197
  _MATERIALIZATIONCONTEXTNODE._serialized_end=3225
  _CONTEXTNODE._serialized_start=3227
  _CONTEXTNODE._serialized_end=3338
# @@protoc_insertion_point(module_scope)
