# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/common/calculation_node__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.common import data_type__client_pb2 as tecton__proto_dot_common_dot_data__type____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n2tecton_proto/common/calculation_node__client.proto\x12\x13tecton_proto.common\x1a+tecton_proto/common/data_type__client.proto\"\x12\n\x10NullLiteralValue\"\x96\x02\n\x0cLiteralValue\x12%\n\rfloat32_value\x18\x01 \x01(\x02H\x00R\x0c\x66loat32Value\x12%\n\rfloat64_value\x18\x02 \x01(\x01H\x00R\x0c\x66loat64Value\x12!\n\x0bint64_value\x18\x03 \x01(\x03H\x00R\nint64Value\x12\x1f\n\nbool_value\x18\x04 \x01(\x08H\x00R\tboolValue\x12#\n\x0cstring_value\x18\x05 \x01(\tH\x00R\x0bstringValue\x12\x46\n\nnull_value\x18\x06 \x01(\x0b\x32%.tecton_proto.common.NullLiteralValueH\x00R\tnullValueB\x07\n\x05value\"\x96\x01\n\tOperation\x12@\n\toperation\x18\x01 \x01(\x0e\x32\".tecton_proto.common.OperationTypeR\toperation\x12G\n\x08operands\x18\x02 \x03(\x0b\x32+.tecton_proto.common.AbstractSyntaxTreeNodeR\x08operands\"\x9c\x01\n\nWhenClause\x12I\n\tcondition\x18\x01 \x01(\x0b\x32+.tecton_proto.common.AbstractSyntaxTreeNodeR\tcondition\x12\x43\n\x06result\x18\x02 \x01(\x0b\x32+.tecton_proto.common.AbstractSyntaxTreeNodeR\x06result\"Q\n\nElseClause\x12\x43\n\x06result\x18\x01 \x01(\x0b\x32+.tecton_proto.common.AbstractSyntaxTreeNodeR\x06result\"\x95\x01\n\rCaseStatement\x12\x42\n\x0cwhen_clauses\x18\x01 \x03(\x0b\x32\x1f.tecton_proto.common.WhenClauseR\x0bwhenClauses\x12@\n\x0b\x65lse_clause\x18\x02 \x01(\x0b\x32\x1f.tecton_proto.common.ElseClauseR\nelseClause\"\x98\x03\n\x16\x41\x62stractSyntaxTreeNode\x12\x33\n\x05\x64type\x18\x01 \x01(\x0b\x32\x1d.tecton_proto.common.DataTypeR\x05\x64type\x12H\n\rliteral_value\x18\x02 \x01(\x0b\x32!.tecton_proto.common.LiteralValueH\x00R\x0cliteralValue\x12+\n\x10\x63olumn_reference\x18\x03 \x01(\tH\x00R\x0f\x63olumnReference\x12>\n\toperation\x18\x04 \x01(\x0b\x32\x1e.tecton_proto.common.OperationH\x00R\toperation\x12<\n\tdate_part\x18\x05 \x01(\x0e\x32\x1d.tecton_proto.common.DatePartH\x00R\x08\x64\x61tePart\x12K\n\x0e\x63\x61se_statement\x18\x06 \x01(\x0b\x32\".tecton_proto.common.CaseStatementH\x00R\rcaseStatementB\x07\n\x05value*\x8e\x02\n\rOperationType\x12\x19\n\x15OPERATION_UNSPECIFIED\x10\x00\x12\x0c\n\x08\x41\x44\x44ITION\x10\x01\x12\x0f\n\x0bSUBTRACTION\x10\x02\x12\x12\n\x0eMULTIPLICATION\x10\x03\x12\x0c\n\x08\x44IVISION\x10\x04\x12\x0c\n\x08\x43OALESCE\x10\x05\x12\r\n\tDATE_DIFF\x10\x06\x12\n\n\x06\x45QUALS\x10\x07\x12\x0e\n\nNOT_EQUALS\x10\x08\x12\r\n\tLESS_THAN\x10\t\x12\x14\n\x10LESS_THAN_EQUALS\x10\n\x12\x10\n\x0cGREATER_THAN\x10\x0b\x12\x17\n\x13GREATER_THAN_EQUALS\x10\x0c\x12\x07\n\x03\x41ND\x10\r\x12\x06\n\x02OR\x10\x0e\x12\x07\n\x03NOT\x10\x0f*\xc9\x01\n\x08\x44\x61tePart\x12\x19\n\x15\x44\x41TE_PART_UNSPECIFIED\x10\x00\x12\x07\n\x03\x44\x41Y\x10\x01\x12\t\n\x05MONTH\x10\x02\x12\x08\n\x04WEEK\x10\x03\x12\x08\n\x04YEAR\x10\x04\x12\n\n\x06SECOND\x10\x05\x12\x08\n\x04HOUR\x10\x06\x12\n\n\x06MINUTE\x10\x07\x12\x0e\n\nMILLENNIUM\x10\x08\x12\x0b\n\x07\x43\x45NTURY\x10\t\x12\n\n\x06\x44\x45\x43\x41\x44\x45\x10\n\x12\x0b\n\x07QUARTER\x10\x0b\x12\x10\n\x0cMILLISECONDS\x10\x0c\x12\x10\n\x0cMICROSECONDS\x10\rB?\n\x11\x63om.tecton.commonP\x01Z(github.com/tecton-ai/tecton_proto/common')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.common.calculation_node__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\021com.tecton.commonP\001Z(github.com/tecton-ai/tecton_proto/common'
  _OPERATIONTYPE._serialized_start=1380
  _OPERATIONTYPE._serialized_end=1650
  _DATEPART._serialized_start=1653
  _DATEPART._serialized_end=1854
  _NULLLITERALVALUE._serialized_start=120
  _NULLLITERALVALUE._serialized_end=138
  _LITERALVALUE._serialized_start=141
  _LITERALVALUE._serialized_end=419
  _OPERATION._serialized_start=422
  _OPERATION._serialized_end=572
  _WHENCLAUSE._serialized_start=575
  _WHENCLAUSE._serialized_end=731
  _ELSECLAUSE._serialized_start=733
  _ELSECLAUSE._serialized_end=814
  _CASESTATEMENT._serialized_start=817
  _CASESTATEMENT._serialized_end=966
  _ABSTRACTSYNTAXTREENODE._serialized_start=969
  _ABSTRACTSYNTAXTREENODE._serialized_end=1377
# @@protoc_insertion_point(module_scope)
