# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/server_groups/server_group_states__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.common import container_image__client_pb2 as tecton__proto_dot_common_dot_container__image____client__pb2
from tecton_proto.common import id__client_pb2 as tecton__proto_dot_common_dot_id____client__pb2
from tecton_proto.common import server_group_status__client_pb2 as tecton__proto_dot_common_dot_server__group__status____client__pb2
from tecton_proto.common import server_group_type__client_pb2 as tecton__proto_dot_common_dot_server__group__type____client__pb2
from tecton_proto.realtime import instance_group__client_pb2 as tecton__proto_dot_realtime_dot_instance__group____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n<tecton_proto/server_groups/server_group_states__client.proto\x12\x1atecton_proto.server_groups\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x31tecton_proto/common/container_image__client.proto\x1a$tecton_proto/common/id__client.proto\x1a\x35tecton_proto/common/server_group_status__client.proto\x1a\x33tecton_proto/common/server_group_type__client.proto\x1a\x32tecton_proto/realtime/instance_group__client.proto\"\xfe\x07\n\x10ServerGroupState\x12J\n\x15server_group_state_id\x18\x0e \x01(\x0b\x32\x17.tecton_proto.common.IdR\x12serverGroupStateId\x12?\n\x0fserver_group_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\rserverGroupId\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name\x12>\n\x06status\x18\x03 \x01(\x0e\x32&.tecton_proto.common.ServerGroupStatusR\x06status\x12\x38\n\x04type\x18\x04 \x01(\x0e\x32$.tecton_proto.common.ServerGroupTypeR\x04type\x12\\\n\x12\x61utoscaling_policy\x18\x11 \x01(\x0b\x32-.tecton_proto.server_groups.AutoscalingPolicyR\x11\x61utoscalingPolicy\x12%\n\x0estatus_details\x18\x05 \x01(\tR\rstatusDetails\x12\x39\n\ncreated_at\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x42\n\x0flast_updated_at\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\rlastUpdatedAt\x12\x1b\n\tmin_nodes\x18\x0f \x01(\x05R\x08minNodes\x12\x1b\n\tmax_nodes\x18\x10 \x01(\x05R\x08maxNodes\x12#\n\rdesired_nodes\x18\r \x01(\x05R\x0c\x64\x65siredNodes\x12x\n\x1ctransform_server_group_state\x18\t \x01(\x0b\x32\x35.tecton_proto.server_groups.TransformServerGroupStateH\x00R\x19transformServerGroupState\x12r\n\x1a\x66\x65\x61ture_server_group_state\x18\n \x01(\x0b\x32\x33.tecton_proto.server_groups.FeatureServerGroupStateH\x00R\x17\x66\x65\x61tureServerGroupState\x12\x1c\n\tworkspace\x18\x0b \x01(\tR\tworkspace\x12\x45\n\x12workspace_state_id\x18\x0c \x01(\x0b\x32\x17.tecton_proto.common.IdR\x10workspaceStateIdB\x13\n\x11server_group_typeJ\x04\x08\x12\x10\x13\"\xd4\x01\n\x11\x41utoscalingPolicy\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12/\n\x13\x61utoscaling_enabled\x18\x02 \x01(\x08R\x12\x61utoscalingEnabled\x12h\n\x16target_tracking_policy\x18\x04 \x01(\x0b\x32\x30.tecton_proto.server_groups.TargetTrackingPolicyH\x00R\x14targetTrackingPolicyB\x10\n\x0escaling_policy\"\x9d\x02\n\x14TargetTrackingPolicy\x12!\n\x0ctarget_value\x18\x01 \x01(\x01R\x0btargetValue\x12*\n\x11scale_in_disabled\x18\x03 \x01(\x08R\x0fscaleInDisabled\x12[\n\x11predefined_metric\x18\x04 \x01(\x0b\x32,.tecton_proto.server_groups.PredefinedMetricH\x00R\x10predefinedMetric\x12O\n\rcustom_metric\x18\x05 \x01(\x0b\x32(.tecton_proto.server_groups.CustomMetricH\x00R\x0c\x63ustomMetricB\x08\n\x06metric\"&\n\x10PredefinedMetric\x12\x12\n\x04type\x18\x01 \x01(\tR\x04type\"T\n\x0c\x43ustomMetric\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x1c\n\tstatistic\x18\x02 \x01(\tR\tstatistic\x12\x12\n\x04unit\x18\x03 \x01(\tR\x04unit\"\xa6\x05\n\x19TransformServerGroupState\x12%\n\x0e\x65nvironment_id\x18\x01 \x01(\tR\renvironmentId\x12)\n\x10\x65nvironment_name\x18\x02 \x01(\tR\x0f\x65nvironmentName\x12\x42\n\nimage_info\x18\x03 \x01(\x0b\x32#.tecton_proto.common.ContainerImageR\timageInfo\x12\x84\x01\n\x15\x65nvironment_variables\x18\x04 \x03(\x0b\x32O.tecton_proto.server_groups.TransformServerGroupState.EnvironmentVariablesEntryR\x14\x65nvironmentVariables\x12\\\n\x12\x61utoscaling_policy\x18\x05 \x01(\x0b\x32-.tecton_proto.server_groups.AutoscalingPolicyR\x11\x61utoscalingPolicy\x12^\n\x15instance_group_handle\x18\x06 \x01(\x0b\x32*.tecton_proto.realtime.InstanceGroupHandleR\x13instanceGroupHandle\x12_\n\x0esecrets_config\x18\x08 \x01(\x0b\x32\x38.tecton_proto.server_groups.TransformServerSecretsConfigR\rsecretsConfig\x1aG\n\x19\x45nvironmentVariablesEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01J\x04\x08\x07\x10\x08\"\xd7\x01\n\x1cTransformServerSecretsConfig\x12,\n\x12service_account_id\x18\x04 \x01(\tR\x10serviceAccountId\x12\x44\n\x1fservice_account_key_secret_name\x18\x02 \x01(\tR\x1bserviceAccountKeySecretName\x12=\n\x0clast_updated\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0blastUpdatedJ\x04\x08\x01\x10\x02\"\xfb\x02\n\x1e\x46\x65\x61tureServerGroupAWSResources\x12\x43\n\x1eloadbalancer_listener_rule_arn\x18\x01 \x01(\tR\x1bloadbalancerListenerRuleArn\x12\x30\n\x14\x61utoscaling_group_id\x18\x02 \x01(\tR\x12\x61utoscalingGroupId\x12(\n\x10target_group_arn\x18\x03 \x01(\tR\x0etargetGroupArn\x12,\n\x12launch_template_id\x18\x04 \x01(\tR\x10launchTemplateId\x12\x43\n\x1elaunch_template_active_version\x18\x05 \x01(\tR\x1blaunchTemplateActiveVersion\x12\x45\n\x1flaunch_template_pending_version\x18\x06 \x01(\tR\x1claunchTemplatePendingVersion\"\xaf\x04\n\x18\x46\x65\x61tureServerGroupConfig\x12#\n\rinstance_type\x18\x01 \x01(\tR\x0cinstanceType\x12(\n\x10\x64ocker_image_fqn\x18\x02 \x01(\tR\x0e\x64ockerImageFqn\x12\x1b\n\thttp_port\x18\x03 \x01(\x05R\x08httpPort\x12\x33\n\x16http_health_check_path\x18\x04 \x01(\tR\x13httpHealthCheckPath\x12!\n\x0cmetrics_port\x18\x05 \x01(\x05R\x0bmetricsPort\x12/\n\x13\x61utoscaling_enabled\x18\x06 \x01(\x08R\x12\x61utoscalingEnabled\x12#\n\rmin_instances\x18\x07 \x01(\x05R\x0cminInstances\x12#\n\rmax_instances\x18\x08 \x01(\x05R\x0cmaxInstances\x12+\n\x11\x64\x65sired_instances\x18\t \x01(\x05R\x10\x64\x65siredInstances\x12g\n\x0b\x65nvironment\x18\n \x03(\x0b\x32\x45.tecton_proto.server_groups.FeatureServerGroupConfig.EnvironmentEntryR\x0b\x65nvironment\x1a>\n\x10\x45nvironmentEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"\xc7\x02\n\x17\x46\x65\x61tureServerGroupState\x12Y\n\ractive_config\x18\x01 \x01(\x0b\x32\x34.tecton_proto.server_groups.FeatureServerGroupConfigR\x0c\x61\x63tiveConfig\x12[\n\x0epending_config\x18\x02 \x01(\x0b\x32\x34.tecton_proto.server_groups.FeatureServerGroupConfigR\rpendingConfig\x12\x61\n\raws_resources\x18\x03 \x01(\x0b\x32:.tecton_proto.server_groups.FeatureServerGroupAWSResourcesH\x00R\x0c\x61wsResourcesB\x11\n\x0f\x63loud_resources*a\n\x11ScalingPolicyType\x12#\n\x1fSCALING_POLICY_TYPE_UNSPECIFIED\x10\x00\x12\'\n#SCALING_POLICY_TYPE_TARGET_TRACKING\x10\x01\x42\x1b\n\x17\x63om.tecton.servergroupsP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.server_groups.server_group_states__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\027com.tecton.servergroupsP\001'
  _TRANSFORMSERVERGROUPSTATE_ENVIRONMENTVARIABLESENTRY._options = None
  _TRANSFORMSERVERGROUPSTATE_ENVIRONMENTVARIABLESENTRY._serialized_options = b'8\001'
  _FEATURESERVERGROUPCONFIG_ENVIRONMENTENTRY._options = None
  _FEATURESERVERGROUPCONFIG_ENVIRONMENTENTRY._serialized_options = b'8\001'
  _SCALINGPOLICYTYPE._serialized_start=4201
  _SCALINGPOLICYTYPE._serialized_end=4298
  _SERVERGROUPSTATE._serialized_start=375
  _SERVERGROUPSTATE._serialized_end=1397
  _AUTOSCALINGPOLICY._serialized_start=1400
  _AUTOSCALINGPOLICY._serialized_end=1612
  _TARGETTRACKINGPOLICY._serialized_start=1615
  _TARGETTRACKINGPOLICY._serialized_end=1900
  _PREDEFINEDMETRIC._serialized_start=1902
  _PREDEFINEDMETRIC._serialized_end=1940
  _CUSTOMMETRIC._serialized_start=1942
  _CUSTOMMETRIC._serialized_end=2026
  _TRANSFORMSERVERGROUPSTATE._serialized_start=2029
  _TRANSFORMSERVERGROUPSTATE._serialized_end=2707
  _TRANSFORMSERVERGROUPSTATE_ENVIRONMENTVARIABLESENTRY._serialized_start=2630
  _TRANSFORMSERVERGROUPSTATE_ENVIRONMENTVARIABLESENTRY._serialized_end=2701
  _TRANSFORMSERVERSECRETSCONFIG._serialized_start=2710
  _TRANSFORMSERVERSECRETSCONFIG._serialized_end=2925
  _FEATURESERVERGROUPAWSRESOURCES._serialized_start=2928
  _FEATURESERVERGROUPAWSRESOURCES._serialized_end=3307
  _FEATURESERVERGROUPCONFIG._serialized_start=3310
  _FEATURESERVERGROUPCONFIG._serialized_end=3869
  _FEATURESERVERGROUPCONFIG_ENVIRONMENTENTRY._serialized_start=3807
  _FEATURESERVERGROUPCONFIG_ENVIRONMENTENTRY._serialized_end=3869
  _FEATURESERVERGROUPSTATE._serialized_start=3872
  _FEATURESERVERGROUPSTATE._serialized_end=4199
# @@protoc_insertion_point(module_scope)
